@@ -1,5 +1,5 @@
 /* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-   Copyright (c) 2016-2022 The plumed team
+   Copyright (c) 2016-2023 The plumed team
    (see the PEOPLE file at the root of the distribution for a list of names)
 
    See http://www.plumed.org for more information.
@@ -109,7 +109,7 @@ public:
     keys.add("compulsory","plumed","plumed.dat","the name of the plumed input file containing the potential");
     keys.add("compulsory","ipos","0.0","the initial position of the system");
     keys.add("compulsory","idum","0","The random number seed");
-    keys.addFlag("periodic","false","are your input coordinates periodic");
+    keys.addFlag("periodic",false,"are your input coordinates periodic");
     keys.add("optional","min","minimum value the coordinates can take for a periodic domain");
     keys.add("optional","max","maximum value the coordinates can take for a periodic domain");
   }
@@ -139,7 +139,7 @@ private:
     parse("tstep",tstep);
     std::string frictionstr; parse("friction",frictionstr);
     if( tempstr!="NVE" ) {
-      if(frictionstr=="off") { std::fprintf(stderr,"Specify friction for thermostat\n"); std::exit(1); }
+      if(frictionstr=="off") error("pecify friction for thermostat");
       Tools::convert(frictionstr,friction);
     }
     parse("plumed",plumedin); parse("dimension",dim);
@@ -218,7 +218,7 @@ public:
     int istep=0; double zero=0;
     plumed->cmd("setStep",&istep);
     plumed->cmd("setMasses",&masses[0]);
-    for(unsigned i=0; i<forces.size(); ++i) forces[i].zero();
+    Tools::set_to_zero(forces);
     plumed->cmd("setForces",&forces[0][0]);
     plumed->cmd("setEnergy",&zero);
     if( lperiod ) plumed->cmd("setBox",&box[0]);
@@ -258,7 +258,7 @@ public:
       plumedWantsToStop=0;
       plumed->cmd("setStep",&istepplusone);
       plumed->cmd("setMasses",&masses[0]);
-      for(unsigned i=0; i<forces.size(); ++i) forces[i].zero();
+      Tools::set_to_zero(forces);
       plumed->cmd("setForces",&forces[0][0]);
       double fenergy=0.0;
       plumed->cmd("setEnergy",&fenergy);
